#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QToolButton>
#include <QAction>
#include <QColorDialog>
#include <QStyleOptionToolButton>
#include <QMenu>
class PaintView;
#include "paintview.h"

class ColorToolButton : public QToolButton {
      Q_OBJECT
    //  Q_PROPERTY(QColor color READ color WRITE setColor NOTIFY colorChanged)
    //  Q_PROPERTY(QString displayText READ displayText WRITE setDisplayText)
public:
    explicit ColorToolButton(QWidget *parent = nullptr)
        : QToolButton(parent), m_color(Qt::transparent) {
        setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        setAutoRaise(true);
    }

    QColor color() const { return m_color; }
    QString displayText() const { return m_text; }

    void setColor(const QColor &color) {
        if (m_color != color) {
            m_color = color;
            update();
            emit colorChanged(m_color);
        }
    }

    void setDisplayText(const QString &text) {
        if (m_text != text) {
            m_text = text;
            update();
        }
    }

    QSize sizeHint() const override {
        QSize base = QToolButton::sizeHint();
        return QSize(base.width() + 30, base.height() + 4);
    }

signals:
    void colorChanged(const QColor &color);

protected:
    void paintEvent(QPaintEvent *event) override {
        Q_UNUSED(event)
        QPainter painter(this);
        QStyleOptionToolButton opt;
        initStyleOption(&opt);

        // 绘制默认按钮背景
        //  style()->drawControl(QStyle::CC_ToolButton, &opt, &painter, this);

        // 绘制颜色块
        QRect colorRect = opt.rect.adjusted(4, 4, -4, -4);
        colorRect.setWidth(opt.rect.height() - 8);
        painter.setPen(Qt::darkGray);
        painter.setBrush(m_color);
        painter.drawRect(colorRect);

        // 绘制文字
        QRect textRect = opt.rect;
        textRect.setLeft(colorRect.right() + 8);
        painter.setPen(opt.palette.color(QPalette::ButtonText));
        painter.drawText(textRect, Qt::AlignLeft | Qt::AlignVCenter, m_text);
    }

private:
    QColor m_color;
    QString m_text;
};

class LineWidthToolButton : public QToolButton {
        Q_OBJECT
public:
    explicit LineWidthToolButton(QWidget *parent = nullptr)
        : QToolButton(parent), m_currentWidth(1) {
        setPopupMode(QToolButton::MenuButtonPopup);
        setToolButtonStyle(Qt::ToolButtonIconOnly);
        buildLineWidthMenu();

        connect(this, &QToolButton::triggered, this, &LineWidthToolButton::updateCurrentWidth);
    }

    int lineWidth() const { return m_currentWidth; }

signals:
    void lineWidthChanged(int width);

protected:
    void paintEvent(QPaintEvent *event) override {
        QToolButton::paintEvent(event);

        // 在按钮上绘制当前线宽
        QPainter painter(this);
        QRect rect = this->rect().adjusted(4, 4, -4, -4);
        painter.setPen(QPen(Qt::black, m_currentWidth));
        painter.drawLine(rect.left(), rect.center().y(),
                         rect.right(), rect.center().y());
    }

private:
    void buildLineWidthMenu() {
        QMenu *menu = new QMenu(this);
        QActionGroup *group = new QActionGroup(menu);

        const QList<int> widths = {1, 2, 4, 6, 10, 12, 16, 20, 22, 34};
        for (int w : widths) {
            QAction *action = new QAction(createLineIcon(w), QString("%1 px").arg(w), menu);
            action->setData(w);
            action->setCheckable(true);
            action->setChecked(w == m_currentWidth);
            group->addAction(action);
            menu->addAction(action);
        }

        setMenu(menu);
    }

    QIcon createLineIcon(int width) {
        QPixmap pixmap(32, 16);
        pixmap.fill(Qt::transparent);

        QPainter painter(&pixmap);
        painter.setRenderHint(QPainter::Antialiasing);
        painter.setPen(QPen(Qt::black, width));
        painter.drawLine(4, 8, 28, 8);

        return QIcon(pixmap);
    }

    void updateCurrentWidth(QAction *action) {
        int width = action->data().toInt();
        if (m_currentWidth != width) {
            m_currentWidth = width;
            update();
            emit lineWidthChanged(width);
        }
    }

    int m_currentWidth;
};

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    QIcon createColorIcon(const QColor &color);
    void createColorToolBar();
    void createColorToolBar2();

    void CreateFirstBar();
    void CreateSecondBar();

    void CreateLineBar();
    ColorToolButton *mpColorBack {nullptr};
    ColorToolButton *mpColorFore {nullptr};
    LineWidthToolButton *mpLineWidth {nullptr};

    void resizeEvent(QResizeEvent *event) override;
private slots:
    void updateRulers();
  //  void handleZoomChanged();
    void SetLineWidth(int value);

private:

    void connectSignals();

    PaintView *m_paintView;
    RulerWidget *m_hRuler;
    RulerWidget *m_vRuler;
    QScrollArea *m_scrollArea;

private slots:
    void open();
    void save();
    void zoomIn();
    void zoomOut();
    void setTool(ToolType tool);
    void updateActions();
    void online();
    void onpen();
    void oncursor();
    void onrect();
    void onellipse();
    void ontext();
    void onerase();
    void ontong();
    void onstroke();
    void undo();
    void redo();
    void clearback();
    void About();

private:
    void createActions();
    void createToolBar();

    QToolBar *m_toolBar;

    QAction *actAbout;

    QAction *actClear;
    QMap<ToolType, QToolButton*> m_toolButtons;
};

#endif // MAINWINDOW_H
