#include "paintview.h"
#include <QPainter>
#include <QMouseEvent>
#include <QUndoCommand>
#include <QInputDialog>
#include <QMessageBox>
#include <QApplication>
#include <QPainterPath>
#include <queue>
#include <QPainterPath>
#include <QScrollBar>
#include <QtMath>

RulerWidget::RulerWidget(Orientation orientation, QWidget *parent)
    : QWidget(parent), m_orientation(orientation)
{
    setFixedSize(m_orientation == Horizontal ? 2000 : 30,
                 m_orientation == Horizontal ? 30 : 2000);
}

void RulerWidget::setOriginOffset(qreal offset) {
    m_originOffset = offset;
    update();
}

void RulerWidget::setZoomFactor(qreal zoom) {
    m_zoomFactor = zoom;
    update();
}

void RulerWidget::setMousePosition(qreal pos) {
    m_mousePos = pos;
    update();
}

void RulerWidget::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    // 绘制背景渐变
    QLinearGradient grad;
    if (m_orientation == Horizontal) {
        grad.setStart(0, height());
        grad.setFinalStop(0, 0);
    } else {
        grad.setStart(width(), 0);
        grad.setFinalStop(0, 0);
    }
    grad.setColorAt(0, QColor(240, 240, 240));
    grad.setColorAt(1, Qt::white);
    painter.fillRect(rect(), grad);

    drawBase(&painter);
    drawTicks(&painter);
    drawNumbers(&painter);
    drawMouseIndicator(&painter);
}

void RulerWidget::drawBase(QPainter *painter) {
    painter->setPen(m_tickPen);
    if (m_orientation == Horizontal) {
        painter->drawLine(0, height()-1, width(), height()-1);
    } else {
        painter->drawLine(width()-1, 0, width()-1, height());
    }
}
// rulerwidget.cpp 改进后的刻度绘制
void RulerWidget::drawTicks(QPainter *painter) {
    const qreal minTickSpacing = 40; // 最小刻度间距(像素)
    const int maxSubTicks = 5;       // 主刻度间最大子刻度数

    // 计算基础单位
    qreal baseUnit = m_metric ? 10.0 : 12.7; // 毫米或0.5英寸
    qreal scaledUnit = baseUnit * m_zoomFactor;

    // 自动确定主刻度间隔
    int power = 0;
    while (scaledUnit * qPow(10, power) < minTickSpacing) power++;
    qreal mainStep = baseUnit * qPow(10, power);
    int subDivisions = qBound(2, static_cast<int>(mainStep / baseUnit), maxSubTicks);
    qreal subStep = mainStep / subDivisions;

    // 计算可见范围
    qreal start = -m_originOffset * m_zoomFactor;
    qreal end = start + (m_orientation == Horizontal ? width() : height());

    // 绘制主刻度
    painter->setPen(QPen(Qt::black, 1.5));
    drawTickLevel(painter, start, end, mainStep, 12);

    // 绘制次刻度
    painter->setPen(QPen(Qt::gray, 0.8));
    drawTickLevel(painter, start, end, subStep, 8);

    // 绘制微刻度（如果空间允许）
    if (subStep * m_zoomFactor > minTickSpacing * 2) {
        painter->setPen(QPen(Qt::lightGray, 0.5));
        drawTickLevel(painter, start, end, subStep/2, 5);
    }
}

void RulerWidget::drawTickLevel(QPainter *painter, qreal start, qreal end,
                                qreal step, qreal tickLength) {
    qreal firstTick = qFloor(start / step) * step;
    qreal lastTick = qCeil(end / step) * step;

    for (qreal pos = firstTick; pos <= lastTick; pos += step) {
        if (pos < start) continue;

        QLineF line;
        if (m_orientation == Horizontal) {
            line = QLineF(pos, height() - 1, pos, height() - tickLength);
        } else {
            line = QLineF(width() - 1, pos, width() - tickLength, pos);
        }

        painter->drawLine(line);
    }
}
// rulerwidget.cpp 实现
qreal RulerWidget::calculateMainStep() const
{
    const qreal minTickSpacing = 40; // 最小主刻度像素间距
    const qreal baseUnit = m_metric ? 10.0 : 12.7; // 公制10mm，英制0.5英寸

    // 动态计算合适的主刻度间隔
    qreal scaledUnit = baseUnit * m_zoomFactor;
    int power = 0;

    // 找到满足最小间距的10进制倍数
    while (scaledUnit * qPow(10, power) < minTickSpacing) {
        power++;
    }

    // 计算实际物理单位间隔
    qreal mainStep = baseUnit * qPow(10, power);

    // 对英制单位进行特殊处理（1/2英寸为基本单位）
    if (!m_metric) {
        if (mainStep > 25.4 * 2) { // 超过2英寸转换为英尺
            mainStep = 25.4 * 12; // 1英尺
        } else if (mainStep > 25.4) { // 1-2英寸保持1英寸间隔
            mainStep = 25.4;
        }
    }

    return mainStep;
}
void RulerWidget::drawNumbers(QPainter *painter) {
    qreal mainStep = calculateMainStep(); // 复用刻度计算逻辑
    qreal firstTick = qFloor((-m_originOffset * m_zoomFactor) / mainStep) * mainStep;
    qreal lastTick = firstTick + (m_orientation == Horizontal ? width() : height()) + mainStep;

    painter->setFont(QFont("Arial", 8, QFont::Bold));
    painter->setPen(Qt::black);

    for (qreal pos = firstTick; pos <= lastTick; pos += mainStep) {
        qreal value = (pos / m_zoomFactor) + m_originOffset;
        QString text = formatUnit(value);

        QRectF rect;
        if (m_orientation == Horizontal) {
            qreal textWidth = painter->fontMetrics().horizontalAdvance(text);
            rect = QRectF(pos - textWidth/2, 2, textWidth, 15);
        } else {
            qreal textWidth = painter->fontMetrics().horizontalAdvance(text);
            rect = QRectF(-9, pos - 10, textWidth, 20);

            if(pos == firstTick)
            {
                rect = QRectF(-2, pos - 10, textWidth, 20);
            }
            painter->save();
            painter->translate(rect.center());
            painter->rotate(-90);
            painter->translate(-rect.center());
        }

        painter->drawText(rect, Qt::AlignCenter, text);
        if (m_orientation == Vertical) painter->restore();
    }
}

QString RulerWidget::formatUnit(qreal value) const {
    if (m_orientation == Vertical) {
        value = qAbs(value); // 垂直标尺显示正值
    }

    if (m_metric) {
        if (value >= 1000) return QString::number(value/1000, 'f', 1) + "m";
        return QString::number(value, 'f', 0) + "cm";
    } else {
        qreal inches = value / 25.4;
        if (inches >= 12) return QString::number(inches/12, 'f', 1) + "'";
        return QString::number(inches, 'f', 1) + "\"";
    }
}

void RulerWidget::drawMouseIndicator(QPainter *painter) {
    if (m_mousePos < 0) return;

    qreal pos = (m_mousePos - m_originOffset) * m_zoomFactor;
    painter->setPen(m_indicatorPen);

    if (m_orientation == Horizontal) {
        painter->drawLine(pos, 0, pos, height());
    } else {
        painter->drawLine(0, pos, width(), pos);
    }
}

void RulerWidget::mouseMoveEvent(QMouseEvent *event) {
    qreal pos = m_orientation == Horizontal
                    ? event->pos().x() / m_zoomFactor + m_originOffset
                    : event->pos().y() / m_zoomFactor + m_originOffset;
   // emit mousePositionChanged(pos);
}
// pathcommand.h
class BaseCommand : public QUndoCommand {
public:
    BaseCommand(QPixmap* pixmap, const QString& text = "")
        : m_pixmap(pixmap), m_initialState(*pixmap), m_done(false) {
        setText(text);
    }

    void redo() override {
        if (!m_done) {
            apply(m_initialState);
            m_done = true;
        } else {
            *m_pixmap = m_modifiedState;
        }
    }

    void undo() override {
        *m_pixmap = m_initialState;
    }

protected:
    virtual void apply(const QPixmap& base) = 0;

    QPixmap* m_pixmap;
    QPixmap m_initialState;
    QPixmap m_modifiedState;
    bool m_done;
};

// 路径绘制命令（钢笔工具）
class PathCommand : public BaseCommand {
public:
    PathCommand(QPixmap* pixmap, const QPainterPath& path,
                const QPen& pen, const QColor& color)
        : BaseCommand(pixmap, "Draw Path"),
        m_path(path), m_pen(pen), m_color(color) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QPainter painter(&m_modifiedState);
        painter.setPen(QPen(m_color, m_pen.width(), Qt::SolidLine, Qt::RoundCap));
        painter.drawPath(m_path);
        *m_pixmap = m_modifiedState;
    }

private:
    QPainterPath m_path;
    QPen m_pen;
    QColor m_color;
};

// 形状绘制命令
class ShapeCommand : public BaseCommand {
public:
    ShapeCommand(QPixmap* pixmap, const QPainterPath& path,
                 const QPen& pen, Qt::BrushStyle fillStyle)
        : BaseCommand(pixmap, "Draw Shape"),
        m_path(path), m_pen(pen), m_fillStyle(fillStyle) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QPainter painter(&m_modifiedState);
        painter.setPen(m_pen);
        if (m_fillStyle != Qt::NoBrush) {
            painter.setBrush(QBrush(m_pen.color(), m_fillStyle));
        }
        painter.drawPath(m_path);
        *m_pixmap = m_modifiedState;
    }

private:
    QPainterPath m_path;
    QPen m_pen;
    Qt::BrushStyle m_fillStyle;
};

// 橡皮擦命令
class EraserCommand : public BaseCommand {
public:
    EraserCommand(QPixmap* pixmap, const QPainterPath& path,
                  int size, const QColor& bgColor)
        : BaseCommand(pixmap, "Erase"),
        m_path(path), m_size(size), m_bgColor(bgColor) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QPainter painter(&m_modifiedState);
        painter.setCompositionMode(QPainter::CompositionMode_Source);
        painter.setPen(QPen(m_bgColor, m_size, Qt::SolidLine, Qt::RoundCap));
        painter.drawPath(m_path);
        *m_pixmap = m_modifiedState;
    }

private:
    QPainterPath m_path;
    int m_size;
    QColor m_bgColor;
};

// 笔触命令
class StrokeCommand : public BaseCommand {
public:
    StrokeCommand(QPixmap* pixmap, const QPolygon& points,
                  const QColor& color, int size)
        : BaseCommand(pixmap, "Brush Stroke"),
        m_points(points), m_color(color), m_size(size) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QPainter painter(&m_modifiedState);
        painter.setRenderHint(QPainter::Antialiasing);
        painter.setPen(QPen(m_color, m_size, Qt::SolidLine, Qt::RoundCap));
        for(int i = 1; i < m_points.size(); ++i) {
            painter.drawLine(m_points[i-1], m_points[i]);
        }
        *m_pixmap = m_modifiedState;
    }

private:
    QPolygon m_points;
    QColor m_color;
    int m_size;
};

// 填充命令
class FillCommand : public BaseCommand {
public:
    FillCommand(QPixmap* pixmap, const QPoint& pos,
                const QColor& oldColor, const QColor& newColor)
        : BaseCommand(pixmap, "Fill Color"),
        m_pos(pos), m_oldColor(oldColor), m_newColor(newColor) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QImage img = m_modifiedState.toImage();
        floodFill(img, m_pos, m_oldColor, m_newColor);
        m_modifiedState.convertFromImage(img);
        *m_pixmap = m_modifiedState;
    }

private:
    void floodFill(QImage& image, QPoint startPos,
                   const QColor& targetColor, const QColor& replacementColor) {
        // 有效性检查
        if (!image.rect().contains(startPos)) return;
        if (targetColor == replacementColor) return;

        // 使用扫描线算法优化填充性能
        std::queue<QPoint> pixelQueue;
        const QRgb targetRGB = targetColor.rgba();
        const QRgb replacementRGB = replacementColor.rgba();

        // 初始像素检查
        if (image.pixel(startPos) != targetRGB) return;

        pixelQueue.push(startPos);

        while (!pixelQueue.empty()) {
            QPoint pt = pixelQueue.front();
            pixelQueue.pop();

            // 找到当前扫描线的最左和最右
            int left = pt.x();
            while (left >= 0 && image.pixel(left, pt.y()) == targetRGB) {
                --left;
            }
            int right = pt.x();
            while (right < image.width() && image.pixel(right, pt.y()) == targetRGB) {
                ++right;
            }

            // 填充当前扫描线
            bool inSpanUp = false;
            bool inSpanDown = false;

            for (int x = left + 1; x < right; ++x) {
                image.setPixel(x, pt.y(), replacementRGB);

                // 检查上方像素
                if (pt.y() > 0) {
                    QRgb upPixel = image.pixel(x, pt.y() - 1);
                    if (!inSpanUp && upPixel == targetRGB) {
                        pixelQueue.push(QPoint(x, pt.y() - 1));
                        inSpanUp = true;
                    } else if (inSpanUp && upPixel != targetRGB) {
                        inSpanUp = false;
                    }
                }

                // 检查下方像素
                if (pt.y() < image.height() - 1) {
                    QRgb downPixel = image.pixel(x, pt.y() + 1);
                    if (!inSpanDown && downPixel == targetRGB) {
                        pixelQueue.push(QPoint(x, pt.y() + 1));
                        inSpanDown = true;
                    } else if (inSpanDown && downPixel != targetRGB) {
                        inSpanDown = false;
                    }
                }
            }
        }
    }

    QPoint m_pos;
    QColor m_oldColor;
    QColor m_newColor;
};

// 填充命令
class PenCommand : public BaseCommand {
public:
    PenCommand(QPixmap* pixmap, const QPoint& pos,
               const QColor& oldColor, const QColor& newColor)
        : BaseCommand(pixmap, "Fill Color"),
        m_pos(pos), m_oldColor(oldColor), m_newColor(newColor) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QImage img = m_modifiedState.toImage();
        floodFill(img, m_pos, m_oldColor, m_newColor);
        m_modifiedState.convertFromImage(img);
        *m_pixmap = m_modifiedState;
    }

private:
    void floodFill(QImage& image, QPoint startPos,
                   const QColor& targetColor, const QColor& replacementColor) {
        // 有效性检查
        if (!image.rect().contains(startPos)) return;
        if (targetColor == replacementColor) return;

        const QRgb targetRGB = targetColor.rgba();
        const QRgb replacementRGB = replacementColor.rgba();

        image.setPixel(startPos.x(), startPos.y(), replacementRGB);
    }

    QPoint m_pos;
    QColor m_oldColor;
    QColor m_newColor;
};

// 文本命令
class TextCommand : public BaseCommand {
public:
    TextCommand(QPixmap* pixmap, const QPoint& pos,
                const QString& text, const QFont& font, const QColor& color)
        : BaseCommand(pixmap, "Add Text"),
        m_pos(pos), m_text(text), m_font(font), m_color(color) {}

protected:
    void apply(const QPixmap& base) override {
        m_modifiedState = base;
        QPainter painter(&m_modifiedState);
        painter.setFont(m_font);
        painter.setPen(m_color);
        painter.drawText(m_pos, m_text);
        *m_pixmap = m_modifiedState;
    }

private:
    QPoint m_pos;
    QString m_text;
    QFont m_font;
    QColor m_color;
};

// class PathCommand : public QUndoCommand {
// public:
//     PathCommand(QPixmap *pixmap, const QPainterPath &path,
//                 const QPen &pen, QObject *parent = nullptr)
//         : QUndoCommand(parent), m_pixmap(pixmap), m_path(path), m_pen(pen)
//     {
//         m_undoPixmap = *m_pixmap;
//     }

//     void undo() override { swap(); }
//     void redo() override { swap(); }

// private:
//     void swap() {
//         QPixmap temp = *m_pixmap;
//         QPainter painter(m_pixmap);
//         painter.drawPixmap(0, 0, m_undoPixmap);
//         if(!m_path.isEmpty()) {
//             painter.setPen(m_pen);
//             painter.drawPath(m_path);
//         }
//         m_undoPixmap = temp;
//     }

//     QPixmap *m_pixmap;
//     QPixmap m_undoPixmap;
//     QPainterPath m_path;
//     QPen m_pen;
// };

// // shapecommand.h
// class ShapeCommand : public QUndoCommand {
// public:
//     ShapeCommand(QPixmap *pixmap, const QPainterPath &shape,
//                  const QPen &pen, QObject *parent = nullptr)
//         : QUndoCommand(parent), m_pixmap(pixmap), m_shape(shape), m_pen(pen)
//     {
//         m_undoPixmap = *m_pixmap;
//     }

//     void undo() override { swap(); }
//     void redo() override { swap(); }

// private:
//     void swap() {
//         QPixmap temp = *m_pixmap;
//         QPainter painter(m_pixmap);
//         painter.drawPixmap(0, 0, m_undoPixmap);
//         painter.setPen(m_pen);
//         painter.drawPath(m_shape);
//         m_undoPixmap = temp;
//     }

//     QPixmap *m_pixmap;
//     QPixmap m_undoPixmap;
//     QPainterPath m_shape;
//     QPen m_pen;
// };

// // 新增橡皮擦命令
// class EraserCommand : public QUndoCommand {
// public:
//     EraserCommand(QPixmap *pixmap, const QPainterPath &path, int size, const QColor &color)
//         : m_pixmap(pixmap), m_path(path), m_size(size), m_color(color)
//     {
//         m_undoPixmap = *m_pixmap;
//     }

//     void undo() override { swap(); }
//     void redo() override { swap(); }

// private:
//     void swap() {
//         QPixmap temp = *m_pixmap;
//         QPainter painter(m_pixmap);
//         painter.drawPixmap(0, 0, m_undoPixmap);
//         m_undoPixmap = temp;
//     }

//     QPixmap *m_pixmap;
//     QPixmap m_undoPixmap;
//     QPainterPath m_path;
//     int m_size;
//     QColor m_color;
// };
// // paintview.cpp 新增实现
// // 新增笔刷命令类
// class StrokeCommand : public QUndoCommand {
// public:
//     StrokeCommand(QPixmap *pixmap, const QVector<QPoint> &points,
//                   const QColor &color, int size)
//         : m_pixmap(pixmap), m_points(points),
//         m_color(color), m_size(size)
//     {
//         m_undoPixmap = *m_pixmap;
//     }

//     void undo() override { swap(); }
//     void redo() override { swap(); }

// private:
//     void swap() {
//         QPixmap temp = *m_pixmap;
//         QPainter painter(m_pixmap);
//         painter.drawPixmap(0, 0, m_undoPixmap);
//         painter.setPen(QPen(m_color, m_size, Qt::SolidLine, Qt::RoundCap));
//         for(int i=1; i<m_points.size(); ++i) {
//             painter.drawLine(m_points[i-1], m_points[i]);
//         }
//         m_undoPixmap = temp;
//     }

//     QPixmap *m_pixmap;
//     QPixmap m_undoPixmap;
//     QVector<QPoint> m_points;
//     QColor m_color;
//     int m_size;
// };
// // 油漆桶命令
// class FillCommand : public QUndoCommand {
// public:
//     FillCommand(QPixmap *pixmap, const QPoint &pos, const QColor &oldColor, const QColor &newColor)
//         : m_pixmap(pixmap), m_pos(pos), m_oldColor(oldColor), m_newColor(newColor)
//     {
//         m_undoPixmap = *m_pixmap;
//     }

//     void undo() override { swap(); }
//     void redo() override { swap(); }

// private:
//     void swap() {
//         QPixmap temp = *m_pixmap;
//         QPainter painter(m_pixmap);
//         painter.drawPixmap(0, 0, m_undoPixmap);
//         m_undoPixmap = temp;
//     }

//     QPixmap *m_pixmap;
//     QPixmap m_undoPixmap;
//     QPoint m_pos;
//     QColor m_oldColor;
//     QColor m_newColor;
// };

PaintView::PaintView(QWidget *parent)
    : QWidget(parent),
    m_currentTool(Cursor),
    m_scaleFactor(5.0), m_backColor(Qt::white)
{
    setAttribute(Qt::WA_StaticContents);
    setMouseTracking(true);
    m_undoStack = new QUndoStack(this);
    m_pen = QPen(Qt::blue, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    m_brush = QBrush(Qt::transparent);
    m_font = QFont("Arial", 12);
    createNewPixmap(600, 500); // 默认新建800x600画布

    m_fillShape = false;
    m_scaleSmooth = true;
    m_antialias = true;
}

// 洪水填充算法实现
void PaintView::floodFill(QImage& image, const QPoint start,
                          const QColor target, const QColor replacement)
{
    // 使用扫描线算法优化性能
    std::queue<QPoint> queue;
    if (image.pixelColor(start) != target) return;

    queue.push(start);
    while (!queue.empty()) {
        QPoint p = queue.front();
        queue.pop();

        int left = p.x();
        int right = p.x();
        while (left >= 0 && image.pixelColor(left, p.y()) == target) --left;
        while (right < image.width() && image.pixelColor(right, p.y()) == target) ++right;

        bool spanUp = false;
        bool spanDown = false;

        for (int x = left + 1; x < right; ++x) {
            image.setPixelColor(x, p.y(), replacement);

            if (p.y() > 0) {
                QColor up = image.pixelColor(x, p.y() - 1);
                if (!spanUp && up == target) {
                    queue.push(QPoint(x, p.y() - 1));
                    spanUp = true;
                } else if (spanUp && up != target) {
                    spanUp = false;
                }
            }

            if (p.y() < image.height() - 1) {
                QColor down = image.pixelColor(x, p.y() + 1);
                if (!spanDown && down == target) {
                    queue.push(QPoint(x, p.y() + 1));
                    spanDown = true;
                } else if (spanDown && down != target) {
                    spanDown = false;
                }
            }
        }
    }
}

void PaintView::createNewPixmap(int width, int height)
{
    m_pixmap = QPixmap(width, height);
    m_pixmap.fill(Qt::white); // 白色背景
    m_scaleFactor = 1.0;
    m_undoStack->clear();
    update();
}

void PaintView::setTool(ToolType tool)
{
    finishTextEditing();
    m_currentTool = tool;
}

void PaintView::setPen(const QPen &pen)
{
    m_pen = pen;
}

void PaintView::setBrush(const QBrush &brush)
{
    m_brush = brush;
}

void PaintView::setFont(const QFont &font)
{
    m_font = font;
}

bool PaintView::openImage(const QString &fileName)
{
    QPixmap newPixmap;
    if (!newPixmap.load(fileName))
        return false;

    m_scaleFactor = 1.0;
    m_pixmap = newPixmap;
    int nWidth = newPixmap.width();
    int nHeight = newPixmap.height();
    int nMax = qMax(nWidth, nHeight);
    if(nMax < 100)
    {
        m_scaleFactor = 5;
    }

    update();
    return true;
}

bool PaintView::saveImage(const QString &fileName)
{
    return m_pixmap.save(fileName);
}
void PaintView::scaleImage(double factor)
{
    finishTextEditing();
    // 限制缩放范围
    const double minFactor = 0.1;
    const double maxFactor = 16.0;
    m_scaleFactor = qBound(minFactor, m_scaleFactor * factor, maxFactor);

    // 仅当需要缩放时更新
    if (qFuzzyCompare(m_scaleFactor, 1.0)) {
        m_scaledPixmap = QPixmap();
    } else {
        // 使用高质量缩放并保持纵横比
        m_scaledPixmap = m_pixmap.scaled(
            m_pixmap.size() * m_scaleFactor,
            Qt::KeepAspectRatio,
            Qt::SmoothTransformation
            );
    }

    if(m_hRuler)
    {
        m_hRuler->setZoomFactor(m_scaleFactor);
        m_hRuler->update();
    }
    if(m_vRuler)
    {
        m_vRuler->setZoomFactor(m_scaleFactor);
        m_vRuler->update();
    }
    update();  // 触发重绘
    //emit scaleChanged();
}

QPixmap PaintView::pixmap() const
{
    return m_pixmap;
}
// 在PaintView中添加方法
void PaintView::setBrushSize(int size)
{
    m_brushSize = qBound(1, size, 50); // 限制1-50像素
}

void PaintView::resizeEvent(QResizeEvent *event)
{
    // 窗口大小变化时重置缩放缓存
    if (!qFuzzyCompare(m_scaleFactor, 1.0)) {
        scaleImage(1.0);  // 保持当前缩放比例但更新缓存
    }

    updateRulers();
    QWidget::resizeEvent(event);
}
void PaintView::mousePressEvent(QMouseEvent* event)
{
    if (m_pixmap.isNull()) return;

    const QPointF canvasPos = scaledPos(event->pos());

    // 文本工具特殊处理
    if (event->button() == Qt::LeftButton && m_currentTool == Text) {
        if (m_isEditingText) {
            // 结束当前编辑
            finishTextEditing();
            return;
        }

        // 开始新文本输入
        startTextEditing(canvasPos);
        return;
    }

    if (event->button() == Qt::RightButton)
    {
        finishTextEditing();
        setTool(Cursor);
        return;
    }

    if (!m_pixmap.rect().contains(canvasPos.toPoint()) && m_currentTool != Bucket)
        return;

    m_beginPos = canvasPos;
    m_lastPos = canvasPos;
    m_isDrawing = true;

    switch (m_currentTool) {
    case Pen:
        m_currentPath = QPainterPath();
        m_currentPath.moveTo(canvasPos);
        break;

    case Eraser:
        m_eraserPath = QPainterPath();
        m_eraserPath.moveTo(canvasPos);
        break;

    case Stroke:
        m_strokePoints.clear();
        m_strokePoints << canvasPos.toPoint();
        break;

    case Line:
    case Rectangle:
    case Ellipse:
        m_tempPath = QPainterPath();
        m_tempPath.moveTo(canvasPos);
        break;

    case Bucket: {
        QImage image = m_pixmap.toImage();
        const QColor oldColor = image.pixelColor(canvasPos.toPoint());
        if (oldColor != m_foreColor) {
            floodFill(image, canvasPos.toPoint(), oldColor, m_foreColor);
            m_pixmap.convertFromImage(image);
            m_undoStack->push(new FillCommand(&m_pixmap, canvasPos.toPoint(), oldColor, m_foreColor));
            update();
        }
        break;
    }
    }
}

void PaintView::mouseMoveEvent(QMouseEvent* event)
{
    if (!m_isDrawing || m_pixmap.isNull()) return;

    const QPointF canvasPos = scaledPos(event->pos());
    const QRectF canvasRect = m_pixmap.rect();
    const bool inCanvas = canvasRect.contains(canvasPos);

    switch (m_currentTool) {
    case Pen: {
        if (inCanvas) {
            m_currentPath.lineTo(canvasPos);
            // 实时绘制到画布
            QPainter painter(&m_pixmap);
            setupPainter(painter);
            painter.drawPath(m_currentPath);
            m_currentPath = QPainterPath();
            m_currentPath.moveTo(canvasPos);
            updateDirtyRect(canvasPos);
        }
        break;
    }

    case Eraser: {
        if (inCanvas) {
            int nNow = m_brushSize;
            if(nNow < 5)
            {
                nNow = 5;
            }

            QPainter painter(&m_pixmap);
            painter.setCompositionMode(QPainter::CompositionMode_Source);
            painter.setPen(QPen(m_backColor, nNow, Qt::SolidLine, Qt::RoundCap));
            painter.drawLine(m_lastPos, canvasPos);
            m_eraserPath.lineTo(canvasPos);
            updateDirtyRect(canvasPos);
        }
        m_lastPos = canvasPos;
        break;
    }

    case Stroke: {
        if (inCanvas) {
            m_strokePoints << canvasPos.toPoint();
            updateDirtyRect(canvasPos);
        }
        break;
    }

    case Line:
    case Rectangle:
    case Ellipse: {
        m_tempPath = QPainterPath();
        buildShape(m_tempPath, m_beginPos, canvasPos);
        update();
        break;
    }
    }
}

void PaintView::mouseReleaseEvent(QMouseEvent* event)
{
    if (m_pixmap.isNull()) return;

    if (m_currentTool == Text) return;

    const QPointF canvasPos = scaledPos(event->pos());
    m_isDrawing = false;

    switch (m_currentTool) {
    case Pen:
    {
        if (!m_currentPath.isEmpty())
        {
            m_undoStack->push(new PathCommand(&m_pixmap, m_currentPath,
                                              QPen(m_foreColor, m_brushSize), m_foreColor));
        }
        else
        {
            QImage image = m_pixmap.toImage();
            const QColor oldColor = image.pixelColor(canvasPos.toPoint());
            if (oldColor != m_foreColor)
            {
                //floodFill(image, canvasPos.toPoint(), oldColor, m_foreColor);
                image.setPixelColor(canvasPos.x(), canvasPos.y(), m_foreColor);
                m_pixmap.convertFromImage(image);
                m_undoStack->push(new PenCommand(&m_pixmap, canvasPos.toPoint(), oldColor, m_foreColor));
                update();
            }
        }
    }


        break;

    case Eraser:
        if (m_eraserPath.elementCount() > 1) {
            int nNow = m_brushSize;
            if(nNow < 5)
            {
                nNow = 5;
            }

            m_undoStack->push(new EraserCommand(&m_pixmap, m_eraserPath,
                                                nNow, m_backColor));
        }
        break;

    case Stroke: {
        if (m_strokePoints.size() > 1) {
            m_undoStack->push(new StrokeCommand(&m_pixmap, QPolygon(m_strokePoints),
                                                m_foreColor, m_brushSize));
        }
        m_strokePoints.clear();
        break;
    }

    case Line:
    case Rectangle:
    case Ellipse: {
        if (!m_tempPath.isEmpty()) {
            Qt::BrushStyle fillStyle = m_fillShape ? Qt::SolidPattern : Qt::NoBrush;
            m_undoStack->push(new ShapeCommand(&m_pixmap, m_tempPath,
                                               QPen(m_foreColor, m_brushSize), fillStyle));
        }
        m_tempPath = QPainterPath();
        break;
    }
    }

    update();
}
// void PaintView::paintEvent(QPaintEvent *event)
// {
//     // 使用分块缓存技术
//     QPixmapCache::setCacheLimit(102400); // 100MB缓存

//     QPainter painter(viewport());
//     const QRect visibleRect = event->rect();

//     if(!QPixmapCache::find("canvas_tile", &m_cachedPixmap)){
//         // 动态生成并缓存画布切片
//         renderCanvasTile();
//         QPixmapCache::insert("canvas_tile", m_cachedPixmap);
//     }

//     painter.drawPixmap(visibleRect, m_cachedPixmap, visibleRect);
// }
// 分块渲染超大画布
//     智能缓存最近访问区域
//         自动释放非可见区域内存

// 绘图视图实现更新（paintview.cpp）
void PaintView::setScale(qreal scale) {
    m_scaleFactor = scale;
    if(m_hRuler)m_hRuler->setZoomFactor(scale);
    if(m_vRuler)m_vRuler->setZoomFactor(scale);
    updateRulers();
    update();
}

void PaintView::updateRulers() {
    const QPointF origin = rect().topLeft();
    if(m_hRuler) m_hRuler->setOriginOffset(origin.x());
    if(m_vRuler) m_vRuler->setOriginOffset(origin.y());

    // QScrollBar* hBar = m_scrollArea->horizontalScrollBar();
    // QScrollBar* vBar = m_scrollArea->verticalScrollBar();
    // m_hRuler->setOffset(hBar->value());
    // m_vRuler->setOffset(vBar->value());
    // QScrollBar* hBar = m_scrollArea->horizontalScrollBar();
    // QScrollBar* vBar = m_scrollArea->verticalScrollBar();

    // m_hRuler->setOffset(hBar->value());
    // m_vRuler->setOffset(vBar->value());
}

void PaintView::drawGrid(QPainter& painter) {
    if (!m_showGrid) return;

    painter.save();
    painter.setPen(QPen(QColor(200, 200, 200), 1, Qt::DotLine));

    const int gridStep = 50 * m_scaleFactor;
    const QRect visibleRect = visibleRegion().boundingRect();

    int startX = visibleRect.left() - (visibleRect.left() % gridStep);
    int startY = visibleRect.top() - (visibleRect.top() % gridStep);

    for (int x = startX; x < visibleRect.right(); x += gridStep) {
        painter.drawLine(x, visibleRect.top(), x, visibleRect.bottom());
    }

    for (int y = startY; y < visibleRect.bottom(); y += gridStep) {
        painter.drawLine(visibleRect.left(), y, visibleRect.right(), y);
    }

    painter.restore();
}

void PaintView::paintEvent(QPaintEvent* event)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setRenderHint(QPainter::SmoothPixmapTransform, true);

    // 绘制背景
    painter.fillRect(rect(), QColor(183,228,240));


    // 绘制主画布
    if (!m_pixmap.isNull()) {
        QPixmap displayPix = m_pixmap.scaled(
            m_pixmap.width() * m_scaleFactor,
            m_pixmap.height() * m_scaleFactor,
            Qt::KeepAspectRatio,
            m_scaleSmooth ? Qt::SmoothTransformation : Qt::FastTransformation
            );

        const QPoint drawPos(
            0,
            0
            );

        painter.drawPixmap(drawPos, displayPix);

        // 绘制实时预览
        painter.save();
        painter.translate(drawPos);
        painter.scale(m_scaleFactor, m_scaleFactor);

        switch (m_currentTool) {
        case Stroke: {
            if (!m_strokePoints.isEmpty()) {
                painter.setPen(QPen(m_foreColor, m_brushSize,
                                    Qt::SolidLine, Qt::RoundCap));
                for (int i = 1; i < m_strokePoints.size(); ++i) {
                    painter.drawLine(m_strokePoints[i-1], m_strokePoints[i]);
                }
            }
            break;
        }
        case Line:
        case Rectangle:
        case Ellipse: {
            if (!m_tempPath.isEmpty()) {
                painter.setPen(QPen(m_foreColor, m_brushSize));
                if (m_fillShape) {
                    painter.setBrush(QBrush(m_foreColor));
                }
                painter.drawPath(m_tempPath);
            }
            break;
        }
        case Pen: {
            if (!m_currentPath.isEmpty()) {
                painter.setPen(QPen(m_foreColor, m_brushSize,
                                    Qt::SolidLine, Qt::RoundCap));
                painter.drawPath(m_currentPath);
            }
            break;
        }
        }
        painter.restore();

        // 绘制所有已保存文本
        painter.save();
        painter.translate(drawPos);
        painter.scale(m_scaleFactor, m_scaleFactor);
        for (const auto& text : m_textItems) {
            painter.setFont(text.font);
            painter.setPen(text.color);
            painter.drawText(text.position, text.content);
        }
        painter.restore();

        // 绘制正在编辑的文本
        if (m_isEditingText) {
            painter.save();
            painter.setFont(m_currentText.font);
            painter.setPen(Qt::gray);

            // 绘制文本输入框
            QRectF textRect = QRectF(positionToWidget(m_currentText.position),
                                     QSizeF(200, 20));
            painter.drawRect(textRect);

            // 绘制预览文本
            if (!m_textEdit->text().isEmpty()) {
                painter.setPen(m_currentText.color);
                painter.drawText(textRect, Qt::AlignLeft|Qt::AlignVCenter,
                                 m_textEdit->text());
            }
            painter.restore();
        }

    }

    drawGrid(painter);
    // 绘制辅助线
    if ((m_currentTool == Line || m_currentTool == Rectangle || m_currentTool == Ellipse)
        && m_isDrawing)
    {
      //  painter.setPen(QPen(Qt::gray, 1, Qt::DashLine));
       // painter.drawRect(QRectF(m_beginPos, scaledPos(QCursor::pos())).normalized());
    }
}
void PaintView::updateDirtyRect(const QPointF& pos)
{
    int nNow = m_brushSize;
    if(nNow < 5)
    {
        nNow = 5;
    }

    const qreal margin = qMax(m_brushSize, nNow) * m_scaleFactor * 2;
    QRectF dirtyRect = QRectF(pos, m_lastPos)
                           .normalized()
                           .adjusted(-margin, -margin, margin, margin);

    update(dirtyRect.toRect());
}
// 辅助方法
QPointF PaintView::scaledPos(const QPoint& pos) const
{
    if (m_pixmap.isNull()) return pos;

    const QSize scaledSize = m_pixmap.size() * m_scaleFactor;
    const QPoint drawOffset(
        0,
        0
        );

    return QPointF(
        (pos.x() - drawOffset.x()) / m_scaleFactor,
        (pos.y() - drawOffset.y()) / m_scaleFactor
        );
}

QPointF PaintView::scaledPos2(const QPoint& pos) const
{
    QPointF widgetPos(pos);
    QSize scaledSize = m_pixmap.size() * m_scaleFactor;
    return QPointF(
        widgetPos.x()/ m_scaleFactor,
        widgetPos.y()/ m_scaleFactor
        );
}

void PaintView::startTextEditing(const QPointF& position)
{
    m_currentText = TextItem{
        position,
        "",
        m_font,       // 从字体设置获取
        m_foreColor   // 使用当前前景色
    };

    // 创建编辑控件
    m_textEdit = new QLineEdit(this);
    connect(m_textEdit, &QLineEdit::editingFinished, this, &PaintView::finishTextEditing);
    m_textEdit->setFont(m_currentText.font);

    // 计算屏幕位置
    QPoint screenPos = canvasToWidgetPosition(position);

  //  QPoint screenPos = mapToGlobal(positionToWidget(position).toPoint());
    m_textEdit->move(screenPos);
   // m_textEdit->setFont(m_font);
   // m_textEdit->setStyleSheet(QString("QLineEdit { color: %1; }").arg(m_foreColor.name()));
    m_textEdit->setStyleSheet(QString(
                                  "QLineEdit { color: %1; background: rgba(255,255,255,200); border: 1px solid gray; }"
                                  ).arg(m_foreColor.name()));

    // 计算最佳控件大小
    const int textWidth = QFontMetrics(m_currentText.font).horizontalAdvance("X") * 40;
    m_textEdit->setFixedWidth(textWidth);

    m_textEdit->show();
    m_textEdit->setFocus();

    m_isEditingText = true;
    update();
}

// void PaintView::startTextEditing(const QPointF& canvasPos)
// {
//     // 计算精确的控件位置
//     const QPoint widgetPos = canvasToWidgetPosition(canvasPos);

//     // 创建编辑控件
//     m_textEdit = new QLineEdit(this);
//     m_textEdit->setAttribute(Qt::WA_ShowWithoutActivating); // 避免自动激活
//     m_textEdit->setWindowFlags(Qt::ToolTip); // 脱离正常布局

//     // 设置字体和颜色
//     m_textEdit->setFont(m_currentText.font);
//     m_textEdit->setStyleSheet(QString(
//                                   "QLineEdit { color: %1; background: rgba(255,255,255,200); border: 1px solid gray; }"
//                                   ).arg(m_foreColor.name()));

//     // 计算最佳控件大小
//     const int textWidth = QFontMetrics(m_currentText.font).horizontalAdvance("X") * 40;
//     m_textEdit->setFixedWidth(textWidth);

//     // 设置位置并显示
//     m_textEdit->move(widgetPos);
//     m_textEdit->show();
//     m_textEdit->setFocus();

//     // 连接信号
//     connect(m_textEdit, &QLineEdit::textChanged, this, &PaintView::updateTextPreview);

//  //   connect(m_textEdit, &QLineEdit::textEdited, this, &PaintView::finishTextEditing);
// }

QPoint PaintView::canvasToWidgetPosition(const QPointF& canvasPos) const
{
    // 获取画布显示区域
    const QPixmap& pixmap = m_pixmap;
    const qreal scale = m_scaleFactor;
    const QSize scaledSize = pixmap.size() * scale;

    // 计算画布在窗口中的显示位置
    const int xOffset = (0) / 2;
    const int yOffset = (0) / 2;

    // 转换为窗口坐标（考虑可能的整数偏移）
    const int x = qRound(xOffset + canvasPos.x() * scale);
    const int y = qRound(yOffset + canvasPos.y() * scale);

    // 确保位置在可见范围内
    return QPoint(
        qBound(0, x, width() - 20),  // 保留右边20像素
        qBound(0, y, height() - 30)  // 保留下边30像素
        );
}

// 新增：实时更新文本预览
void PaintView::updateTextPreview()
{
    m_currentText.content = m_textEdit->text();
    update(); // 触发重绘预览
}

void PaintView::finishTextEditing()
{
    if (!m_textEdit) return;

    m_currentText.content = m_textEdit->text();
    if (!m_currentText.content.isEmpty()) {

        m_undoStack->push(new TextCommand(&m_pixmap, m_currentText.position.toPoint(),
                                              m_currentText.content,m_currentText.font,
                                              m_currentText.color));
        m_textItems.append(m_currentText);
    }

    m_textEdit->deleteLater();
    m_isEditingText = false;
    update();
}

QPointF PaintView::positionToWidget(const QPointF& canvasPos) const
{
    QSize scaledSize = m_pixmap.size() * m_scaleFactor;
    return QPointF(
        (0)/2 + canvasPos.x()*m_scaleFactor,
        (0)/2 + canvasPos.y()*m_scaleFactor
        );
}

void PaintView::setupPainter(QPainter& painter)
{
    painter.setRenderHint(QPainter::Antialiasing, m_antialias);
    painter.setPen(QPen(m_foreColor, m_brushSize,
                        Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));

    if (m_currentTool == Eraser) {
        painter.setCompositionMode(QPainter::CompositionMode_Source);
    }
}

void PaintView::setTextFont(const QFont& font)
{
    m_font = font;
    if (m_textEdit) {
        m_textEdit->setFont(font);
    }
}

void PaintView::setTextColor(const QColor& color)
{
    m_foreColor = color; // 共享前景色设置
    if (m_textEdit) {
        m_textEdit->setStyleSheet(QString("QLineEdit { color: %1; }")
                                      .arg(color.name()));
    }
}
// paintview.cpp 中的绘图方法
void PaintView::drawEraser(QPainter &painter, const QPoint &from, const QPoint &to)
{
    int nNow = m_brushSize;
    if(nNow < 5)
    {
        nNow = 5;
    }
    painter.save();
    painter.setCompositionMode(QPainter::CompositionMode_Clear);
    painter.setPen(QPen(Qt::transparent, nNow,
                        Qt::SolidLine, Qt::RoundCap));
    painter.drawLine(from, to);
    painter.restore();
}

void PaintView::buildShape(QPainterPath& path, QPointF start, QPointF end)
{
    path = QPainterPath();

    switch (m_currentTool) {
    case Line:
        path.moveTo(start);
        path.lineTo(end);
        break;
    case Rectangle:
        path.addRect(QRectF(start, end));
        break;
    case Ellipse:
        path.addEllipse(QRectF(start, end));
        break;
    }
}

QPoint PaintView::scaledPosInv(const QPoint& imagePos) const
{
    if (m_scaledPixmap.isNull()) return imagePos;

    // 计算窗口坐标
    QPoint pixmapOffset(
        (0) / 2,
        (0) / 2
        );

    return QPoint(
        qRound(imagePos.x() * m_scaleFactor) + pixmapOffset.x(),
        qRound(imagePos.y() * m_scaleFactor) + pixmapOffset.y()
        );
}

void PaintView::setBack(const QColor& color)
{
    m_backColor = color;
    if (!m_pixmap.isNull()) {
        m_pixmap.fill(color);
        update();
    }
}

void PaintView::drawPreview(QPainter &painter)
{
    if ((m_currentTool != Cursor) &&
        (QApplication::mouseButtons() & Qt::LeftButton))
    {
        painter.setPen(QPen(m_pen.color(), m_pen.width(), Qt::DotLine));
        painter.setBrush(Qt::NoBrush);

        QRect previewRect = QRect(m_beginPos.toPoint(), m_lastPos.toPoint()).normalized();

        switch (m_currentTool) {
        case Line:
            painter.drawLine(m_beginPos, m_lastPos);
            break;
        case Rectangle:
            painter.drawRect(previewRect);
            break;
        case Ellipse:
            painter.drawEllipse(previewRect);
            break;
        case Text:
            painter.drawRect(previewRect);
            break;
        default:
            break;
        }
    }
}

QUndoStack *PaintView::undoStack() const
{
    return m_undoStack;
}
