#ifndef PAINTVIEW_H
#define PAINTVIEW_H

#include <QWidget>
#include <QLineEdit>
#include <QTextItem>
#include <QPointer>
class RulerWidget : public QWidget {
    Q_OBJECT
public:
    enum Orientation { Horizontal, Vertical };

    RulerWidget(Orientation orientation, QWidget *parent = nullptr);

    void setOriginOffset(qreal offset);
    void setZoomFactor(qreal zoom);
    void setMousePosition(qreal pos);
    void setUnitSystem(bool useMetric);
    void drawTickLevel(QPainter *painter, qreal start, qreal end,
                                    qreal step, qreal tickLength);

    qreal calculateMainStep() const;
protected:
    void paintEvent(QPaintEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;

private:
    void drawBase(QPainter *painter);
    void drawTicks(QPainter *painter);
    void drawNumbers(QPainter *painter);
    void drawMouseIndicator(QPainter *painter);
    QString formatUnit(qreal value) const;

    Orientation m_orientation;
    qreal m_originOffset = 0;
    qreal m_zoomFactor = 1.0;
    qreal m_mousePos = -1;
    bool m_metric = true;
    QPen m_tickPen = QPen(Qt::black, 1);
    QPen m_indicatorPen = QPen(Qt::red, 1, Qt::DotLine);
};

enum ToolType {
    Cursor,
    Point,
    Line,
    Rectangle,
    Ellipse,
    Text,
    Eraser,
    Bucket,
    Stroke, // 新增自由绘制工具
    Pen
};
#include <QWidget>
#include <QUndoStack>
#include <QGraphicsView>
#include <QPainterPath>

class PaintView : public QWidget
{
    Q_OBJECT
public:
    explicit PaintView(QWidget *parent = nullptr);

    void setTool(ToolType tool);
    void setPen(const QPen &pen);
    void setBrush(const QBrush &brush);
    void setFont(const QFont &font);
    QColor foregroundColor() const { return m_foreColor; }
    QColor backgroundColor() const { return m_backColor; }
    void setForegroundColor(const QColor &color) { m_foreColor = color; }
    void setBackgroundColor(const QColor &color) { m_backColor = color; }
    void setBrushSize(int size);
    bool openImage(const QString &fileName);
    bool saveImage(const QString &fileName);
    void createNewPixmap(int width, int height);
    void scaleImage(double factor);
    QUndoStack *undoStack() const;

    QPixmap pixmap() const;
    void floodFill(QImage& image, const QPoint start,
                              const QColor target, const QColor replacement);
    void setBack(const QColor& color);
    // 新增成员变量
    QColor m_foreColor = Qt::black;
    QColor m_backColor = Qt::white;
    int m_eraserSize = 10;
    QPainterPath m_eraserPath;
    QPainterPath m_currentPath;
    QPainterPath m_tempPath;
    QPixmap m_scaledPixmap;
    QPixmap m_tempPixmap;
    bool m_isDrawing {false};
    // 形状填充控制
    bool m_fillShape = false;  // 是否填充形状（默认不填充）

    // 缩放质量控制
    bool m_scaleSmooth = true; // 缩放时使用平滑算法（默认开启）

    // 抗锯齿控制
    bool m_antialias = true;   // 启用抗锯齿渲染（默认开启）
    void drawEraser(QPainter& painter, const QPoint& from, const QPoint& to);
    // 新增笔刷属性
    int m_brushSize = 3;
    QVector<QPoint> m_strokePoints;
    QPointF scaledPos(const QPoint &pos) const;
    QPoint scaledPosInv(const QPoint& imagePos) const;
    void setupPainter(QPainter& painter);
    void buildShape(QPainterPath& path, QPointF start, QPointF end);
    void updateDirtyRect(const QPointF& pos);

    void setScale(qreal scale);
    void setGridVisible(bool visible);

    void drawGrid(QPainter& painter);

public:
    bool m_showGrid = true;
    QScrollArea* m_scrollArea;
    RulerWidget* m_hRuler {nullptr};
    RulerWidget* m_vRuler {nullptr};

    void updateRulers();

signals:
    void scaleChanged(double factor);
public:
    // 新增文本相关成员
    struct TextItem {
        QPointF position;
        QString content;
        QFont font;
        QColor color;
    };
    QList<TextItem> m_textItems;
    TextItem m_currentText;
    QPointer<QLineEdit> m_textEdit;
    bool m_isEditingText = false;
    QPointF positionToWidget(const QPointF& canvasPos) const;
    void setTextFont(const QFont& font);
    void setTextColor(const QColor& color);
    void startTextEditing(const QPointF& position);
    QPointF scaledPos2(const QPoint& pos) const;
    QPoint canvasToWidgetPosition(const QPointF& canvasPos) const;
private slots:
    void finishTextEditing();
    void updateTextPreview();
protected:
    void paintEvent(QPaintEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void resizeEvent(QResizeEvent *event) override;
public:
    QPixmap m_pixmap;
    QPointF m_lastPos;
    QPointF m_beginPos;
    ToolType m_currentTool;
    ToolType m_currentShape;
    QPen m_pen;
    QBrush m_brush;
    QFont m_font;
    QUndoStack *m_undoStack;
    double m_scaleFactor;

    void drawPreview(QPainter &painter);

    friend class PaintCommand;
};
#include <QPainter>
#include <QMouseEvent>
#include <QUndoCommand>

class PaintCommand : public QUndoCommand
{
public:
    PaintCommand(PaintView *view, const QPixmap &oldPixmap)
        : m_view(view), m_oldPixmap(oldPixmap), m_newPixmap(view->pixmap()) {}

    void undo() override { m_view->m_pixmap = m_oldPixmap; m_view->update(); }
    void redo() override { m_view->m_pixmap = m_newPixmap; m_view->update(); }

private:
    PaintView *m_view;
    QPixmap m_oldPixmap;
    QPixmap m_newPixmap;
};

#endif // PAINTVIEW_H
